%
% Call this function on each byte b received from the ThinkGear serial stream.
%
% Returns -3 if the parser is not in SYNC with the serial stream yet.  (Usually safe to ignore)
% Returns -2 if b completes a Packet, but the Packet's checksum is bad.
% Returns 0 if b is in the middle of a Packet.
% Returns 1 if b completes a valid Packet.
%
function [ this parse_result, payload ] = parseByte(this, b )

    %persistent this.automaton;
    if isempty( this.automaton ) || isstr( b)
        this.automaton.parser_state = 1;
        this.automaton.parser_pLength = 0;
        this.automaton.parser_pBytesReceived = 0;
        this.automaton.parser_payload = []; %initialize an array that can contain up to 256 bytes
        this.automaton.parser_pSum = 0;
        this.automaton.parser_chksum = 0;
        if isstr( b ), return; end;
    end;
    
    payload = [];
    parse_result = 0;
    
    % Act on b based on current parser this.automaton...
    switch( this.automaton.parser_state )

        % SYNC1
        case 1,
            if( b == 170 ) this.automaton.parser_state = 2; % go to SYNC2
            else parse_result = -3; end;

        % SYNC2
        case 2,
            if( b == 170 ) 
                this.automaton.parser_state = 3; % go to PLENGTH
            else 
                parse_result = -3;
                this.automaton.parser_state = 1; % back to SYNC1
            end;

        % PLENGTH
        case 3,
            if b > 40, 
                disp('Corrupted packet detected'); 
                this.automaton.parser_state = 1;
            else
                this.automaton.parser_pLength = b;
                if (this.automaton.parser_pLength == 0) || this.automaton.parser_pLength > 100
                    fprintf('Packet length %d, out of sync signal, resync\n', this.automaton.parser_pLength);
                    this.automaton.parser_state = 1; % resync
                else
                    this.automaton.parser_pBytesReceived = 0;
                    this.automaton.parser_pSum = 0;
                    this.automaton.parser_state = 4; % go to PAYLOAD
                end;
            end;

        % PAYLOAD
        case 4,
            this.automaton.parser_pBytesReceived = this.automaton.parser_pBytesReceived + 1;
            this.automaton.parser_payload( this.automaton.parser_pBytesReceived ) = b;
            this.automaton.parser_pSum = this.automaton.parser_pSum + b;
            while( this.automaton.parser_pSum > 256 ) this.automaton.parser_pSum = this.automaton.parser_pSum - 256; end;
            if( this.automaton.parser_pBytesReceived == this.automaton.parser_pLength )
                this.automaton.parser_state = 5; % go to CHKSUM
            end;

        % CHKSUM
        case 5,
            this.automaton.parser_chksum = b;
            this.automaton.parser_state = 1;  % back to SYNC1
            % If payload passes chksum...
            if( (256-this.automaton.parser_pSum-1) == this.automaton.parser_chksum )
                % TODO: safe to parse payload and use its data values now
                parse_result = 1;
                payload = this.automaton.parser_payload(1:this.automaton.parser_pBytesReceived);
            % Else payload failed checksum...
            else
                %disp('Warning: packet integrity lost due to discontinuity in acquisition');
                parse_result = -2;
            end;

    end; % "Switch on this.automaton.parser_this.automaton...."
