% LOADINFO load information from text files defining connection formats
%
%    serialinfo = LOADINFO(filaname)
%
%    filename - the filename contining the serial port information 
%
%    serialinfo - this output structure contains the serial connection
%                 information from the data file.

% Copyright (c) 2008-2009 NeuroSky, Inc. All Rights Reserved

function structure = loadinfo( filename);

fid = fopen(filename, 'r');
if fid == -1
    error('Cannot open file');
end;

structure = [];
maxcount = 1;
while ~feof(fid)
    str = fgetl(fid);
    indc = find(str == ':');
    if ~isempty(indc)
        field = deblank(str(1:indc-1));
        value = str(indc+1:end);
        count = 1;
        
        % decode the value
        % ----------------
        while ~isempty(value)
            [t value] = strtok(value);
            tnum = str2double(t); if ~isnan(tnum), t = tnum; end;
            tmpstruct = structure;
            if ~isempty(t)
                if isstr(t) % string
                    structure = setfield(structure, lower(field), {count}, {t});
                    count = count+1;
                else % numeric
                    for tmpi = 1:length(t)
                        structure = setfield(structure, lower(field), {count+tmpi-1}, t(tmpi));
                    end;
                    count = count+length(t);
                end;
                maxcount = max(maxcount, count-1);
            end;
        end;
    end;
end;
fclose(fid);

% deal with different options for single thinkgear
% ------------------------------------------------
if length(structure.thinkgear) < maxcount
    tmpthinkgear = structure.thinkgear{1};
    for index = 1:maxcount
        structure.thinkgear{index} = [ tmpthinkgear '_' num2str(structure.streaming_speed(index)) 'bd' ];
    end;
end;
        
fields = fieldnames(structure);
% fill in if some values are missing
% and convert everything to cell arrays
for f = 1:length(fields)
    lenfield = length(getfield(structure, fields{f}));
    if strcmpi(fields{f}, 'headset')
        tmpval    = getfield(structure, fields{f});
        structure = setfield(structure, fields{f}, tmpval{1});        
    else
        for tmpi = lenfield+1:maxcount
            tmpval    = getfield(structure, fields{f}, {1});
            structure = setfield(structure, fields{f}, {tmpi}, tmpval);
        end;
    end;
end;

        