function nskout = loadstreamlogfile(filename, varargin);

% decode parameters
opt = struct(varargin{:});
if ~isfield(opt, 'srate'),         opt.srate      = 128; end;
if ~isfield(opt, 'filepos'),       opt.filepos    = 100000; end;
if ~isfield(opt, 'save'),          opt.save       = 'off'; end;
if ~isfield(opt, 'loadnsk'),       opt.loadnsk    = 'on'; end;
if ~isfield(opt, 'decodevolt'),    opt.decodevolt = 'on'; end;
if ~isfield(opt, 'headset'),       opt.headset    = 'mindset'; end;
if ~isfield(opt, 'headsetoption'), opt.headsetoption  = 1; end;

% load nsk file if present
filenamensk = [ filename(1:end-4) '_nsk.mat' ];
if strcmpi(opt.loadnsk, 'on')
    if exist(filenamensk)
        tmp = load('-mat', filenamensk);
        if isfield('nsk', tmp)
            nskout = nsk;
            return;
        else
            disp('Matlab file does not contain presaved nsk variable - must be An''s old format');
        end;
    end;
end;

clear functions;
clear global streambuffer parsedpackets nsk;

% load a data file -> output global var streambuffer
if ~strcmpi(filename(end-2:end), 'mat')

    % autodetect file type
    if isempty(opt.headset)
        detectedsrate = detectstreamsrate(filename);
        fprintf('Detected sampling rate: %3.2f\n', detectedsrate);
    
        [tmp ind] = min(abs([128 256 512]-detectedsrate));
        if ind == 1
            disp('Mindset - Atmel chip detected (128 hz)');
            opt.headset = 'mindset';
            opt.headsetoption = 3; % atmel
            opt.srate   = 128;
        elseif ind == 2
            disp('SSVEP - FPGA 256 Hz detected');
            opt.headset = 'ssvep';
            opt.headsetoption = 1;
            opt.srate   = 256;
        elseif ind == 3
            disp('Mindset - ASIC chip detected (512 Hz)');
            opt.headset = 'mindset';
            opt.headsetoption = 1; %ASIC
            opt.srate   = 512;
        end;
    end;
    
    filestream('file', filename, 'srate', opt.srate, 'filepos', opt.filepos, 'streambuffervar', 'streambuffer', ...
               'headset', opt.headset, 'headsetoption', opt.headsetoption);   
else
    tmpobj = headsetinfo('headset', opt.headset, 'headsetoption', opt.headsetoption); 
    global streambuffer;
    streambuffer.params  = tmpobj.params;
    a = load('-mat', filename);
    streambuffer.data    = a.dataall';
    streambuffer.pointer = length(a.dataall)+1;
end;
disp('File read, now decoding packets...');

% take streambuffer as input and output parsedpackets
if strcmpi( opt.headset, 'ssvep'), parsertype = 'FPGA'; else parsertype = 'normal'; end;
tmpobj = packetparser('parsertype', parsertype, 'streambuffervar', 'streambuffer'); packetparser_process(tmpobj);

% take parsedpackets and output nsk
disp('Packets read, now decoding data...');
tmpobj2 = datarowdecoder('srate', opt.srate, 'decodevolt', opt.decodevolt); datarowdecoder_process(tmpobj2);

% remove extra data from the nsk variable
global nsk
fields = fieldnames(nsk);
fields = setdiff(fields, 'params');
for i = 1:length(fields)
    eval( [ 'count = nsk.' fields{i} '(1).count;' ]);
    try, eval( [ 'nsk.' fields{i} '(1).values = nsk.' fields{i} '(1).values(:,1:count-1);' ]); catch, end;
    try, eval( [ 'nsk.' fields{i} '(1).voltvalues = nsk.' fields{i} '(1).voltvalues(:,1:count-1);' ]); catch, end;
    try, eval( [ 'nsk.' fields{i} '(1).time   = nsk.' fields{i} '(1).time(1:count-1);' ]); catch, end;
end;
if isfield(nsk, 'flash')
    for i=1:length(nsk.flash)
        nsk.flash(i).timevalues = nsk.flash(i).timevalues(1:nsk.flash(i).count-1);
        nsk.flash(i).datacount  = nsk.flash(i).datacount(1:nsk.flash(i).count-1);
    end;    
end;
nskout = nsk;

% save the data file
if strcmpi(opt.save, 'on')
    filename = [ filename(1:end-4) '_nsk.mat' ];
    save('-mat', filename, 'nsk');
end;
