% SELECTINFO select information from structure
%
%    params = SELECTINFO(allformats, headset, options);
%
%    allformats - the structure containing all connection information
%                 from the mindsetinfo folder
%    headset    - headset in structure above
%    options    - integer containing the option for a given headset
%
%    params - this output structure contains the serial connection
%             information from the data file.

% Copyright (c) 2008-2009 NeuroSky, Inc. All Rights Reserved

function params = selectinfo(allformats, headset, options);

    allnames = lower({ allformats.headset });
    headset = strmatch(lower(num2str(headset)), allnames);
    if isempty(headset)
        allformatname = '';
        for tmpind = 1:length(allnames)
            allformatname = [ allformatname ' ''' allnames{tmpind} '''' ];
        end;
        error(['Unknown format name''' headset ''', allowed names are ' allformatname ]);
    end;
    
    params = [];
    params.headset         = getfield( allformats, { headset }, 'headset');
    params.thinkgear       = getfield( allformats, { headset }, 'thinkgear'     ,  { options });
    params.opening_speed   = getfield( allformats, { headset }, 'opening_speed',   { options });
    params.streaming_speed = getfield( allformats, { headset }, 'streaming_speed', { options });
    try, params.lower_bit_level = getfield( allformats, { headset }, 'lower_bit_level' ,{ options }); catch, params.lower_bit_level = NaN; end;
    try, params.upper_bit_level = getfield( allformats, { headset }, 'upper_bit_level', { options }); catch, params.upper_bit_level = NaN; end;
    try, params.gain            = getfield( allformats, { headset }, 'gain',            { options }); catch, params.gain = NaN; end;
    try, params.sampling_rate   = getfield( allformats, { headset }, 'sampling_rate',   { options }); catch, params.sampling_rate = 128; end;
    try, params.volt_reference  = getfield( allformats, { headset }, 'volt_reference',  { options }); catch, params.volt_reference = 3.3; end;    
    params.thinkgear       = params.thinkgear{1};

    params.connect_command_1 = [];
    params.connect_command_2 = [];
    try, params.connect_command_1 = getfield( allformats, { headset }, 'connect_command_1', { options }); catch, end;
    try, params.connect_command_2 = getfield( allformats, { headset }, 'connect_command_2', { options }); catch, end;
