% POP_LOADSTREAMLOGFILE load information from Neurosky stream log text
%                       file
%
%    nsk = POP_LOADSTREAMLOGFILE           % pop up window
%    nsk = POP_LOADSTREAMLOGFILE(filename)
%
%    filename - the filename contining the serial port information 
%
%    nsk - this output structure contains the output data.

% Copyright (c) 2008-2009 NeuroSky, Inc. All Rights Reserved

function EEG = pop_loadstreamlogfile(filename, varargin);

EEG = [];
if nargin < 1
    [tmpf tmpp] = uigetfile('*.txt', 'Select Neurosky stream log file');
    if tmpf(1) == 0, return; end;
    filename = fullfile(tmpp, tmpf);
end;

if ~exist('loadstreamlogfile')
    tmpp = which('pop_loadstreamlogfile');
    addpath(fullfile(tmpp(1:end-23), 'nskimportfiles'));
end;
nsk = loadstreamlogfile(filename, varargin{:});
EEG = eeg_emptyset;
if isfield(nsk, 'data')
    EEG.data   = nsk.data.values;
    EEG.srate  = nsk.data.srate;
else
    for ind = 1:7
        fdname = [ 'EEG_RAW' int2str(ind) ];
        if isfield(nsk, fdname)
            if ind == 1, EEG.data        = nsk.EEG_RAW1.values;
            else         EEG.data(ind,:) = getfield(nsk, fdname, 'values');
            end;
        end;
    end;
    EEG.srate  = nsk.params.sampling_rate;
end;
EEG.nbchan = size(EEG.data,1);
EEG = eeg_checkset(EEG);
