% pop_loadthinkgear() - load Neurosky thinkgear file
%
% Usage:
%   >> [EEG, com] = pop_loadthinkgear; % pop-up window mode
%   >> [EEG, com] = pop_loadthinkgear(filename);
%
% Optional inputs:
%   filename  - file name
%
% Outputs:
%   EEG       - EEGLAB EEG structure
%   com       - history string

% Copyright (c) 2008-2009 NeuroSky, Inc. All Rights Reserved

function [EEG, data, com] = pop_loadthinkgear(filename, varargin)

    com = '';
    data = [];
    EEG = [];

    % not enough arguments -> pop-up a window
    % ---------------------------------------
    if nargin < 1
        % pop-up window for file name
        % ---------------------------
        [hdrfile path] = uigetfile2('*.txt', 'Select thinkgear file - pop_loadthinkgear()');
        if hdrfile(1) == 0, return; end
        filename = fullfile(path, hdrfile);
        
        cont = true;
        while cont == true
            % pop-up window for basic interface: type of information to import
            % ----------------------------------------------------------------
            cb_advanced = 'set(gcbf, ''userdata'', ''advanced''); set(findobj(gcbf, ''tag'', ''ok''), ''userdata'', ''retuninginputui'');';
            cb_epoch    = 'set(findobj(gcbf, ''tag'', ''event''), ''value'', 0);';
            cb_event    = 'set(findobj(gcbf, ''tag'', ''epoch''), ''value'', 0);';
            
            uilist = { { 'style' 'text'       'string' strvcat('What to import?', ' ', ' ') 'fontweight' 'bold'  } ...
                       { 'style' 'listbox'    'string' 'Import channel continuous data and events|Import power and e-sensors' } ...
                       };
%                      {} { } { 'style' 'pushbutton' 'string' 'Click here for advanced options' 'callback' cb_advanced} { } };
            %uigeom = { [1 2] [1] [0.2 1 0.2] };
            uigeom = { [1 2]};
            uivert = [2.6];
            [ res userdat ] = inputgui('geometry', uigeom, 'uilist', uilist, 'geomvert', uivert);
            if isempty(res), return; end;
            
            if ~isstr(userdat), 
                cont = false;
            else
                % pop-up window for advanced interface
                % ------------------------------------
                uilist = { { 'style' 'text'     'string' 'What to import?' 'fontweight' 'bold' } ...
                           { 'style' 'checkbox' 'string' 'Import data channel'         'value' 1 } ...
                           { } { 'style' 'checkbox' 'string' 'Data format as 1-sec epochs' 'value' 0 'tag' 'epoch' 'callback' cb_epoch } ...
                           { 'style' 'checkbox' 'string' 'Import frequency band power' } ...
                           { 'style' 'checkbox' 'string' 'Import e-sensors' } ...
                           { 'style' 'checkbox' 'string' 'Import batery level' } ...
                           { } ...
                           { 'style' 'text'     'string' 'Events associated with the data:' 'fontweight' 'bold' } ...
                           { 'style' 'checkbox' 'string' 'Import epoch limits'        'value' 1 'tag' 'event' 'callback' cb_event } ...
                           { 'style' 'checkbox' 'string' 'Import other events if any' 'value' 1 } ...
                           { 'style' 'checkbox' 'string' 'Import old thinkgear data format (-2008)' 'value' 0 } ...
                           { } ...
                           { } { 'style' 'pushbutton' 'string' 'Click here for basic options' 'callback' cb_advanced} { }};
                uigeom = { 1 1 [0.1 1]  1 1 1 1 1 1 1 1 [0.1 1 0.1] };
                uivert = [1 1 1 1 1 1 1 1 1 1 0.5 1 ];
                [res userdat] = inputgui('geometry', uigeom, 'uilist', uilist, 'geomvert', uivert);
                if isempty(res), return; end;
                if ~isstr(userdat), cont = false; end;
            end;
        end;
        
        % decode inputs
        % -------------
        if length(res) > 6 % advanced options
            options = { };
            if ~res{1}, options = { options{:} 'importdata'        'off' }; end;
            if  res{2}, options = { options{:} 'dataasepoch'       'on'  }; end;
            if  res{3}, options = { options{:} 'importpower'       'on'  }; end;
            if  res{4}, options = { options{:} 'importesense'      'on'  }; end;
            if  res{5}, options = { options{:} 'importbattery'     'on'  }; end;
            if ~res{6}, options = { options{:} 'importepochevents' 'off' }; end;
            if ~res{7}, options = { options{:} 'importeventfile'   'off' }; end;
            if  res{8}, options = { options{:} 'importoldformat'   'on'  }; end;
        else % basic options
            options = { };
            %  res{1} == 1 correspond to default options
            if res{1} == 2, options = { options{:} 'importdata' 'off' 'importepochevents' 'off' 'importeventfile' 'off' 'importpower' 'on' 'importesense' 'on' }; end;
            if res{1} == 3, options = { options{:} 'importpower' 'on' 'importesense' 'on' }; end;
        end;
    else
        options = varargin;
    end

    % decode options
    % --------------
    opt = finputcheck( options, { 'importdata'        'string' { 'on' 'off' } 'on' ;
                                  'importpower'       'string' { 'on' 'off' } 'off';
                                  'importbattery'     'string' { 'on' 'off' } 'off';
                                  'importepochevents' 'string' { 'on' 'off' } 'on' ;
                                  'importesense'      'string' { 'on' 'off' } 'off';
                                  'dataasepoch'       'string' { 'on' 'off' } 'off';                                  
                                  'importeventfile'   'string' { 'on' 'off' } 'on' ;
                                  'importoldformat'   'string' { 'on' 'off' } 'off' }, 'pop_loadthinkgear');
    if isstr(opt), error(opt); return; end;
    
    % import data file
    % ----------------
    if exist([ filename(1:end-4) '_events.txt'] ) & strcmpi(opt.importeventfile, 'on')
         disp('Found event file');
         if strcmpi(opt.importoldformat, 'on')
              data = neurosky_read_thinkgearold(filename, [ filename(1:end-4) '_events.txt']);
         else data = neurosky_read_thinkgear(   filename, [ filename(1:end-4) '_events.txt']);
         end;
    elseif exist([ filename(1:end-4) '-events.txt'] ) & strcmpi(opt.importeventfile, 'on')
         disp('Found event file');
         if strcmpi(opt.importoldformat, 'on')
              data = neurosky_read_thinkgearold(filename, [ filename(1:end-4) '-events.txt']);
         else data = neurosky_read_thinkgear(   filename, [ filename(1:end-4) '-events.txt']);
         end;
    else
         if strcmpi(opt.importoldformat, 'on')
              data = neurosky_read_thinkgearold(filename);
         else data = neurosky_read_thinkgear(   filename);
         end;
    end;
    
    % create EEG dataset
    % ------------------
    EEG          = eeg_emptyset;
    EEG.srate    = data.srate;
    EEG.data     = [];
    EEG.chanlocs = [];
    if strcmpi(opt.importdata, 'on'),
        if strcmpi(opt.dataasepoch, 'on')
            % data import in epoch format
            EEG.data = zeros(1, 128, length(data.databounds));
            for index = 1:length(data.databounds)
                EEG.data(1,:,index) = data.valuesvolt(data.databounds(index):data.databounds(index)+127);
            end
            
            % import all epoch information in the time locking event at 0
            for index = 1:length(data.databounds)
                EEG.event(index).type    = 'TLE';
                EEG.event(index).latency   = data.databounds(index)-data.databounds(1)+1;
                if ~isempty(data.power), 
                    EEG.event(index).delta_power  = data.power(1,index);
                    EEG.event(index).theta_power  = data.power(2,index);
                    EEG.event(index).alpha1_power = data.power(3,index);
                    EEG.event(index).alpha2_power = data.power(4,index);
                    EEG.event(index).beta1_power  = data.power(5,index);
                    EEG.event(index).beta2_power  = data.power(6,index);
                end;
                if ~isempty(data.e_sense), 
                    EEG.event(index).attention  = data.e_sense(1,index);
                    EEG.event(index).meditation = data.e_sense(2,index);
                end;
                if ~isempty(data.battery), 
                    EEG.event(index).battery = data.battery(1,index);
                end;
                EEG.event(index).epoch = index;
            end;
        else
            % data import, only power/esense and/or battery
            EEG.data   = data.valuesvolt'; 
            EEG.chanlocs.labels = 'Data channel';
            if strcmpi(opt.importpower, 'on'), 
                power        = filluparray(data.power, data.databounds, length(data.valuesint));
                EEG.data     = [ EEG.data; power ];
                EEG.chanlocs = struct('labels', { 'Data channel' 'Delta' 'Theta' 'Alpha1' 'Alpha2' 'Beta1' 'Beta2' 'Low-gamma' 'Mid-gamma' });
                EEG.chanlocs(size(EEG.data,1)+1:end) = [];
            end;
            if strcmpi(opt.importesense, 'on'), 
                esense   = filluparray(data.e_sense, data.databounds, length(data.valuesint));
                EEG.data = [ EEG.data; esense ];
                EEG.chanlocs(end+1).labels = 'Attention';
                EEG.chanlocs(end+1).labels = 'Meditation';
            end;
            if strcmpi(opt.importbattery, 'on'), 
                battery      = filluparray(data.battery, data.databounds, length(data.valuesint));
                EEG.data     = [ EEG.data; battery ];
                EEG.chanlocs(end+1).labels = 'Battery';
            end;
        end;
    else
        % no data import, only power/esense and/or battery
        % time axis is only 1 point per second
        EEG.srate = 1;
        if strcmpi(opt.importpower, 'on'),
            if ~isempty(data.power)
                EEG.data = data.power;
                EEG.chanlocs = struct('labels', { 'Delta' 'Theta' 'Alpha1' 'Alpha2' 'Beta1' 'Beta2' 'Low-gamma' 'Mid-gamma'});
                EEG.chanlocs(size(EEG.data,1)+1:end) = [];
            else
                disp('Warning: this data file does not contain power information');
            end;
        end;
        if strcmpi(opt.importesense, 'on'), 
            EEG.data = [ EEG.data; data.e_sense ];
            EEG.chanlocs(end+1).labels = 'Attention';
            EEG.chanlocs(end+1).labels = 'Meditation';
        end;
        if strcmpi(opt.importbattery, 'on'), 
            EEG.data = [ EEG.data; data.battery ];
            EEG.chanlocs(end+1).labels = 'Battery';
        end;
    end;
    
    EEG.nbchan = size(EEG.data, 1);
    EEG.pnts   = size(EEG.data, 2);
    EEG.trials = size(EEG.data, 3);
    EEG.xmin   = 0;
    EEG.xmax   = (EEG.pnts - 1) / EEG.srate;

    % import events
    % -------------
    if isfield(data, 'event') & strcmpi(opt.importeventfile, 'on')
        for index = 1:length(data.event)
            EEG.event(end+1).type    = data.event(index).type;
            EEG.event(end).latency   = data.event(index).latency;
            
            if strcmpi(opt.dataasepoch, 'on')
                tmplat = data.databounds-data.event(index).latency;
                tmplat = find(tmplat >= 0);
                EEG.event(end).latency = EEG.event(end).latency - data.databounds(1)+1;
                EEG.event(end).epoch   = tmplat(1); % first index to be positive
            end;
        end;
    end;
    
    % import epoch limits
    % -------------------
    if length(data.databounds) >= 2
        epochlen = unique([data.databounds(2:end)-data.databounds(1:end-1)]);
        if length(epochlen) > 1, disp('Warning: inconsistent epoch length'); end;
        if strcmpi(opt.importepochevents, 'on'), 
            for index = 1:length(data.databounds)
                EEG.event(end+1).type    = 'epoch limit';
                EEG.event(end).latency   = data.databounds(index);
            end
            [tmp tmpind] = sort([ EEG.event.latency ]);
            EEG.event = EEG.event(tmpind);
        end;
    end;
    
    EEG = eeg_checkset(EEG);
    
    % output text for history
    % -----------------------
    if ~isempty(options)
         com = sprintf('EEG = pop_loadthinkgear(''%s'', %s);', filename, vararg2str(options));
    else com = sprintf('EEG = pop_loadthinkgear(''%s'');'    , filename);
    end;
    
% function to coregister power and esensor with raw data
% ------------------------------------------------------
function newarray = filluparray(values, indices, newarraylength)

% check input sizes
if size(values,2) ~= length(indices), values = values'; end;
if size(values,2) ~= length(indices)
    error('The number of values does not correspond to the number of indices');
end;

% copy values
newarray = zeros(size(values, 1), newarraylength);
indices  = [ indices(:)' newarraylength ];
for i = 1:length(indices)-1
    newarray(:,indices(i):indices(i+1)-1) = repmat(values(:,i), [ 1 indices(i+1)-indices(i) ]);
end;
    
