%%
% Parses @c payload into an array of @c datarows (CODE + value) according to the ThinkGear
% Packet specification.
%
function [datarows] = parse_payload( this, payload )

    payload_length = length( payload );
    payload_index = 1;

    datarows = {};
    datarow_index = 1;

    % light in FPGA board
    % -------------------
    if payload_length == 2 && payload(payload_index) > 240
        datarows = { payload };
        return;
    end;
        
    % While there are more DataRows in the packet to process...
    while( payload_index < payload_length )

        % Determine the EXCODE level of this DataRow's CODE by counting
        % the number of EXCODE bytes preceding the CODE
        % (EXCODE bytes are bytes of value 0x55; CODE is never 0x55)
        datarow_excode_level = 0;
        while( payload(payload_index) == 85 )
            datarow_excode_level = datarow_excode_level + 1;
            payload_index = payload_index + 1;
        end

        % Determine the CODE of this DataRow
        datarow_code = payload( payload_index );
        payload_index = payload_index + 1;

        % Determine the number of bytes comprising this DataRow's value
        % (CODEs less than 0x80 are single-byte values, while CODEs greater
        % than or equal to 0x80 are multi-byte values.  For multi-byte values, the
        % length of the multi-byte value immediately follows the CODE)
        if( datarow_code >= 128 )
            value_length = payload( payload_index );
            datarows(datarow_index) = { payload(payload_index-1:(payload_index + value_length)) };
            payload_index = payload_index + value_length + 1;
        else
            datarows(datarow_index) = { payload(payload_index-1:payload_index) };
            payload_index = payload_index + 1;
        end

        % Done with the current DataRow
        datarow_index = datarow_index + 1;
       
    end % "Process each DataRow..."
